"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Api = require("./api");
const Utils = require("./utils");
// Global option values.
let safeMode;
let htmlReplacement;
let callback;
// Reset options to default values.
function init() {
    safeMode = 0;
    htmlReplacement = "<mark>replaced HTML</mark>";
    callback = undefined;
}
exports.init = init;
// Return true if safeMode is non-zero.
function isSafeModeNz() {
    return safeMode !== 0;
}
exports.isSafeModeNz = isSafeModeNz;
function getSafeMode() {
    return safeMode;
}
exports.getSafeMode = getSafeMode;
// Return true if Macro Definitions are ignored.
function skipMacroDefs() {
    /* tslint:disable:no-bitwise */
    return safeMode !== 0 && (safeMode & 0x8) === 0;
    /* tslint:enable:no-bitwise */
}
exports.skipMacroDefs = skipMacroDefs;
// Return true if Block Attribute elements are ignored.
function skipBlockAttributes() {
    /* tslint:disable:no-bitwise */
    return (safeMode & 0x4) !== 0;
    /* tslint:enable:no-bitwise */
}
exports.skipBlockAttributes = skipBlockAttributes;
function setSafeMode(value) {
    let n = Number(value);
    if (isNaN(n) || n < 0 || n > 15) {
        errorCallback("illegal safeMode API option value: " + value);
        return;
    }
    safeMode = n;
}
function setHtmlReplacement(value) {
    if (value === undefined)
        return;
    htmlReplacement = value;
}
function setReset(value) {
    if (value === false || value === "false") {
        return;
    }
    else if (value === true || value === "true") {
        Api.init();
    }
    else {
        errorCallback("illegal reset API option value: " + value);
    }
}
function updateOptions(options) {
    for (let key in options) {
        switch (key) {
            case "reset":
            case "safeMode":
            case "htmlReplacement":
            case "callback":
                break;
            default:
                errorCallback("illegal API option name: " + key);
                return;
        }
    }
    if ("callback" in options)
        callback = options.callback; // Install callback first to ensure option errors are logged.
    if ("reset" in options)
        setReset(options.reset); // Reset takes priority.
    if ("callback" in options)
        callback = options.callback; // Install callback again in case it has been reset.
    if ("safeMode" in options)
        setSafeMode(options.safeMode);
    if ("htmlReplacement" in options) {
        setHtmlReplacement(options.htmlReplacement);
    }
}
exports.updateOptions = updateOptions;
// Set named option value.
function setOption(name, value) {
    let option = {};
    option[name] = value;
    updateOptions(option);
}
exports.setOption = setOption;
// Filter HTML based on current safeMode.
function htmlSafeModeFilter(html) {
    /* tslint:disable:no-bitwise */
    switch (safeMode & 0x3) {
        /* tslint:enable:no-bitwise */
        case 0: // Raw HTML (default behavior).
            return html;
        case 1: // Drop HTML.
            return "";
        case 2: // Replace HTML with 'htmlReplacement' option string.
            return htmlReplacement;
        case 3: // Render HTML as text.
            return Utils.replaceSpecialChars(html);
        default:
            return "";
    }
}
exports.htmlSafeModeFilter = htmlSafeModeFilter;
function errorCallback(message) {
    if (callback) {
        callback({ type: "error", text: message });
    }
}
exports.errorCallback = errorCallback;
// Called when an unexpected program error occurs.
function panic(message) {
    let msg = "panic: " + message;
    console.error(msg);
    errorCallback(msg);
}
exports.panic = panic;
