"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const DelimitedBlocks = require("./delimitedblocks");
const Macros = require("./macros");
const Options = require("./options");
const Spans = require("./spans");
// http://stackoverflow.com/questions/3561493/is-there-a-regexp-escape-function-in-javascript
function escapeRegExp(s) {
    return s.replace(/[\-\/\\^$*+?.()|\[\]{}]/g, "\\$&");
}
exports.escapeRegExp = escapeRegExp;
function replaceSpecialChars(s) {
    return s.replace(/&/g, "&amp;")
        .replace(/>/g, "&gt;")
        .replace(/</g, "&lt;");
}
exports.replaceSpecialChars = replaceSpecialChars;
// Replace pattern '$1' or '$$1', '$2' or '$$2'... in `replacement` with corresponding match groups
// from `match`. If pattern starts with one '$' character add specials to `expansionOptions`,
// if it starts with two '$' characters add spans to `expansionOptions`.
function replaceMatch(match, replacement, expansionOptions = {}) {
    return replacement.replace(/(\${1,2})(\d)/g, function () {
        // Replace $1, $2 ... with corresponding match groups.
        if (arguments[1] === "$$") {
            expansionOptions.spans = true;
        }
        else {
            expansionOptions.specials = true;
        }
        let i = Number(arguments[2]); // match group number.
        let result = match[i]; // match group text.
        if (result === undefined) {
            Options.errorCallback("undefined replacement group: " + arguments[0]);
            return "";
        }
        return replaceInline(result, expansionOptions);
    });
}
exports.replaceMatch = replaceMatch;
// Shallow object clone.
function copy(source) {
    let result = {};
    for (let key in source) {
        if (source.hasOwnProperty(key)) {
            result[key] = source[key];
        }
    }
    return result;
}
exports.copy = copy;
// Copy properties in source object to target object.
function merge(target, source) {
    for (let key in source) {
        target[key] = source[key];
    }
}
exports.merge = merge;
// Replace the inline elements specified in options in text and return the result.
function replaceInline(text, expansionOptions) {
    if (expansionOptions.macros) {
        text = Macros.render(text);
    }
    // Spans also expand special characters.
    if (expansionOptions.spans) {
        text = Spans.render(text);
    }
    else if (expansionOptions.specials) {
        text = replaceSpecialChars(text);
    }
    return text;
}
exports.replaceInline = replaceInline;
// Global Block Attributes state (namespace "singleton", see http://stackoverflow.com/a/30174360).
var BlockAttributes;
(function (BlockAttributes) {
    let ids // List of allocated HTML ids.
    ;
    function init() {
        BlockAttributes.classes = "";
        BlockAttributes.id = "";
        BlockAttributes.css = "";
        BlockAttributes.attributes = "";
        BlockAttributes.options = {};
        ids = [];
    }
    BlockAttributes.init = init;
    function parse(match) {
        // Parse Block Attributes.
        // class names = $1, id = $2, css-properties = $3, html-attributes = $4, block-options = $5
        let text = match[0];
        text = replaceInline(text, { macros: true });
        let m = /^\\?\.((?:\s*[a-zA-Z][\w\-]*)+)*(?:\s*)?(#[a-zA-Z][\w\-]*\s*)?(?:\s*)?(?:"(.+?)")?(?:\s*)?(\[.+])?(?:\s*)?([+-][ \w+-]+)?$/
            .exec(text);
        if (!m) {
            return false;
        }
        if (!Options.skipBlockAttributes()) {
            if (m[1]) { // HTML element class names.
                BlockAttributes.classes += " " + m[1].trim();
                BlockAttributes.classes = BlockAttributes.classes.trim();
            }
            if (m[2]) { // HTML element id.
                BlockAttributes.id = m[2].trim().slice(1);
            }
            if (m[3]) { // CSS properties.
                if (BlockAttributes.css && BlockAttributes.css.substr(-1) !== ";")
                    BlockAttributes.css += ";";
                BlockAttributes.css += " " + m[3].trim();
                BlockAttributes.css = BlockAttributes.css.trim();
            }
            if (m[4] && !Options.isSafeModeNz()) { // HTML attributes.
                BlockAttributes.attributes += " " + m[4].slice(1, m[4].length - 1).trim();
                BlockAttributes.attributes = BlockAttributes.attributes.trim();
            }
            DelimitedBlocks.setBlockOptions(BlockAttributes.options, m[5]);
        }
        return true;
    }
    BlockAttributes.parse = parse;
    // Inject HTML attributes from attrs into the opening tag.
    // Consume HTML attributes unless the 'tag' argument is blank.
    function inject(tag, consume = true) {
        if (!tag) {
            return tag;
        }
        let attrs = "";
        if (BlockAttributes.classes) {
            let re = /^(<[^>]*class=")(.*?)"/i;
            if (re.test(tag)) {
                // Inject class names into first existing class attribute in first tag.
                tag = tag.replace(re, `$1${BlockAttributes.classes} $2"`);
            }
            else {
                attrs = `class="${BlockAttributes.classes}"`;
            }
        }
        if (BlockAttributes.id) {
            BlockAttributes.id = BlockAttributes.id.toLowerCase();
            let has_id = /^<[^<]*id=".*?"/i.test(tag);
            if (has_id || ids.indexOf(BlockAttributes.id) > -1) {
                Options.errorCallback(`duplicate 'id' attribute: ${BlockAttributes.id}`);
            }
            else {
                ids.push(BlockAttributes.id);
            }
            if (!has_id) {
                attrs += ` id="${BlockAttributes.id}"`;
            }
        }
        if (BlockAttributes.css) {
            let re = /^(<[^>]*style=")(.*?)"/i;
            if (re.test(tag)) {
                // Inject CSS styles into first existing style attribute in first tag.
                tag = tag.replace(re, function (match, p1, p2) {
                    p2 = p2.trim();
                    if (p2 && p2.substr(-1) !== ";")
                        p2 += ";";
                    return `${p1}${p2} ${BlockAttributes.css}"`;
                });
            }
            else {
                attrs += ` style="${BlockAttributes.css}"`;
            }
        }
        if (BlockAttributes.attributes) {
            attrs += " " + BlockAttributes.attributes;
        }
        attrs = attrs.trim();
        if (attrs) {
            let match = tag.match(/^<([a-zA-Z]+|h[1-6])(?=[ >])/);
            if (match) {
                let before = tag.slice(0, match[0].length);
                let after = tag.slice(match[0].length);
                tag = before + " " + attrs + after;
            }
        }
        // Consume the attributes.
        if (consume) {
            BlockAttributes.classes = "";
            BlockAttributes.id = "";
            BlockAttributes.css = "";
            BlockAttributes.attributes = "";
        }
        return tag;
    }
    BlockAttributes.inject = inject;
    function slugify(text) {
        let slug = text.replace(/\W+/g, "-") // Replace non-alphanumeric characters with dashes.
            .replace(/-+/g, "-") // Replace multiple dashes with single dash.
            .replace(/(^-)|(-$)/g, "") // Trim leading and trailing dashes.
            .toLowerCase();
        if (!slug)
            slug = "x";
        if (ids.indexOf(slug) > -1) { // Another element already has that id.
            let i = 2;
            while (ids.indexOf(slug + "-" + i) > -1) {
                i++;
            }
            slug += "-" + i;
        }
        return slug;
    }
    BlockAttributes.slugify = slugify;
})(BlockAttributes = exports.BlockAttributes || (exports.BlockAttributes = {}));
